#!/usr/bin/env python

"A WebStack questionnaire application."

import WebStack.Generic
from XSLForms.Resources.WebResources import \
    XSLFormsResource, input, output, resources, prepare_resources as xslforms_prepare_resources
import os
import libxml2dom
import xml.dom

# Site map imports.

from WebStack.Resources.ResourceMap import MapResource
from WebStack.Resources.Selectors import EncodingSelector
from WebStack.Resources.Static import DirectoryResource

# Configuration settings.

encoding = "utf-8"

# Resource classes.

class QuestionnaireEditorResource(XSLFormsResource):

    "A resource providing a questionnaire editor."

    resource_dir = resources(__file__)
    template_resources = {
        "questionnaire" : output("question_template.xhtml")
        }
    init_resources = {
        "questionnaire" : input("question_template.xhtml")
        }
    document_resources = {
        "response-types" : "question_response_types.xml"
        }

    def select_activity(self, trans, form):
        form.set_activity("questionnaire")

    def respond_to_input(self, trans, form):

        parameters = form.get_parameters()
        selectors = form.get_selectors()
        questionnaire = form.get_document()

        self.remove_elements(selectors.get("remove-question"))
        self.add_elements(selectors.get("add-choice"), "choice")
        self.remove_elements(selectors.get("remove-choice"))

        # Move questions.
        # NOTE: Potentially a DOM convenience method here.

        for question in selectors.get("move-question", []):
            destination = int(question.getAttribute("destination"))
            if destination > 1:
                destination += 1
            root = questionnaire.documentElement
            target = (root.xpath("question[position() = %d]" % destination) or [None])[0]
            root.removeChild(question)
            if target is not None:
                try:
                    root.insertBefore(question, target)
                except xml.dom.NotFoundErr:
                    target = (root.xpath("question[position() = %d]" % destination) or [None])[0]
                    root.insertBefore(question, target)
            else:
                root.appendChild(question)

        # Add questions using the normal request parameter.

        if parameters.has_key("add-question"):
            new_question = questionnaire.ownerDocument.createElementNS(None, "question")
            questionnaire.xpath("questionnaire")[0].appendChild(new_question)

        # Send a response according to certain parameters.
        # When exported, an XML version of the data is returned.

        if parameters.has_key("export"):
            trans.set_content_type(WebStack.Generic.ContentType("text/xml", encoding))
            questionnaire.toStream(trans.get_response_stream(), trans.get_response_stream_encoding())
            raise WebStack.Generic.EndOfResponse

        # Support uploading of files.

        if parameters.has_key("import"):
            importfile = parameters["importfile"][0]
            try:
                doc = libxml2dom.parse(importfile.stream)
                form.set_document(doc)
            except libxml2dom.LSException:
                pass

    def init_document(self, trans, form):

        # Transform, adding enumerations/ranges.

        response_types_xml = self.prepare_document("response-types")
        XSLFormsResource.init_document(self, trans, form, references={"response-type" : response_types_xml})

# Site map initialisation.

def get_site():

    "Return a simple Web site resource."

    # Get the main resource and the directory used by the application.

    questionnaire_resource = QuestionnaireEditorResource()
    directory = questionnaire_resource.resource_dir

    # Make a simple Web site.

    resource = MapResource({
        # Static resources:
        "styles" : DirectoryResource(os.path.join(directory, "styles"), {"css" : "text/css"}),
        # Main page:
        "" : questionnaire_resource
        })

    return EncodingSelector(resource, encoding)

# Resource preparation ahead of time - useful for making installations.

def prepare_resources():
    for cls in [QuestionnaireEditorResource]:
        xslforms_prepare_resources(cls)

# vim: tabstop=4 expandtab shiftwidth=4
